/* 
 * Copyright (C) 2012 FoxLabs
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.foxlabs.validation.constraint;

import java.util.Map;
import java.util.Set;
import java.util.Collections;

import java.net.URL;
import java.net.MalformedURLException;


import org.foxlabs.util.Assert;
import org.foxlabs.validation.ValidationContext;

/**
 * This class provides <code>CheckConstraint</code> implementation that checks
 * whether a string is valid URL reference. Also the allowed set of protocols
 * can be configured.
 * 
 * @author Fox Mulder
 * @see Url
 * @see ConstraintFactory#url()
 * @see ConstraintFactory#url(String...)
 */
public final class UrlConstraint extends CheckConstraint<String> {
    
    /**
     * <code>UrlConstraint</code> default instance initialized with empty set
     * of protocols.
     */
    public static final UrlConstraint DEFAULT = new UrlConstraint((String[]) null);
    
    /**
     * Set of allowed protocols (empty set means all protocols are allowed).
     */
    private final Set<String> protocols;
    
    /**
     * Constructs a new <code>UrlConstraint</code> with the specified array of
     * allowed protocols.
     * 
     * @param protocols Array of allowed protocols.
     * @throws IllegalArgumentException if the specified array of protocols
     *         contains <code>null</code> or empty elements.
     */
    UrlConstraint(String[] protocols) {
        this.protocols = protocols == null || protocols.length == 0
            ? Collections.<String>emptySet()
            : Collections.unmodifiableSet(Assert.noEmptyStringSet(protocols, "protocols"));
    }
    
    /**
     * Constructs a new <code>UrlConstraint</code> from the specified
     * annotation.
     * 
     * @param annotation Constraint annotation.
     * @throws IllegalArgumentException if the specified annotation defines
     *         array of protocols that contains empty elements.
     */
    UrlConstraint(Url annotation) {
        this(annotation.protocols());
    }
    
    /**
     * Returns <code>java.lang.String</code> type.
     * 
     * @return <code>java.lang.String</code> type.
     */
    @Override
    public Class<?> getType() {
        return String.class;
    }
    
    /**
     * Returns set of allowed protocols (empty set means all protocols are
     * allowed).
     * 
     * @return Set of allowed protocols.
     */
    public Set<String> getProtocols() {
        return protocols;
    }
    
    /**
     * Appends <code>protocols</code> argument that contains set of allowed
     * protocols.
     * 
     * @param context Validation context.
     * @param arguments Arguments to be substituted into the error message
     *        template.
     * @return <code>true</code>.
     */
    @Override
    public boolean appendMessageArguments(ValidationContext<?> context, Map<String, Object> arguments) {
        super.appendMessageArguments(context, arguments);
        if (protocols.size() > 0)
            arguments.put("protocols", protocols);
        return true;
    }
    
    /**
     * Checks whether the specified string is valid URL reference with allowed
     * protocol.
     * 
     * @param value URL string.
     * @param context Validation context.
     * @return <code>true</code> if the specified string is valid URL reference
     *         with allowed protocol; <code>false</code> otherwise.
     */
    @Override
    protected <T> boolean check(String value, ValidationContext<T> context) {
        if (value == null)
            return true;
        try {
            URL url = new URL(value);
            return protocols.isEmpty() || protocols.contains(url.getProtocol().toLowerCase());
        } catch (MalformedURLException e) {}
        return false;
    }
    
}
